@@ -1,5 +1,5 @@
 #
-#   Copyright 2007-2016 The OpenMx Project
+#   Copyright 2007-2017 The OpenMx Project
 #
 #   Licensed under the Apache License, Version 2.0 (the "License");
 #   you may not use this file except in compliance with the License.
@@ -292,6 +292,18 @@ model <- mxModel(model, mxAlgebra(dnbinom(D,D,-1,C,C),name="test84b"))
 model <- mxModel(model, mxAlgebra(pnbinom(D,D,-1,C,C,0),name="test85a"))
 model <- mxModel(model, mxAlgebra(pnbinom(D,D,-1,C,1,0),name="test85b"))
 model <- mxModel(model, mxAlgebra(pnbinom(D,D,-1,C,0,1),name="test85c"))
+model <- mxModel(model, mxAlgebra(dchisq(A,A,A,A),name="test86a"))
+model <- mxModel(model, mxAlgebra(pchisq(A,A,A,A,A),name="test86b"))
+model <- mxModel(model, mxAlgebra(dchisq(D,1,-1,0),name="test87a"))
+model <- mxModel(model, mxAlgebra(pchisq(D,1,-1,0,0),name="test87b"))
+model <- mxModel(model, mxAlgebra(dbinom(D,D,A,0),name="test88a"))
+model <- mxModel(model, mxAlgebra(dbinom(D,D,A,1),name="test88b"))
+model <- mxModel(model, mxAlgebra(pbinom(D,D,A,0,0),name="test89a"))
+model <- mxModel(model, mxAlgebra(pbinom(D,D,A,1,1),name="test89b"))
+model <- mxModel(model, mxAlgebra(dcauchy(H,A,D,0),name="test90a"))
+model <- mxModel(model, mxAlgebra(dcauchy(H,A,D,1),name="test90b"))
+model <- mxModel(model, mxAlgebra(pcauchy(H,A,D,0,1),name="test91a"))
+model <- mxModel(model, mxAlgebra(pcauchy(H,A,D,1,0),name="test91b"))
 model <- mxRun(model)
 
 # Check passing tests
@@ -557,6 +569,20 @@ omxCheckCloseEnough(model[['test85b']]$result, pnbinom(q=D$values,size=D$values,
 																											 log.p=FALSE), .001)
 omxCheckCloseEnough(model[['test85c']]$result, pnbinom(q=D$values,size=D$values,mu=C$values[1:10],lower.tail=FALSE,
 																											 log.p=TRUE), .001)
+omxCheckCloseEnough(model[['test86a']]$result, dchisq(x=A$values,df=A$values,ncp=A$values,log=TRUE), .001)
+omxCheckCloseEnough(model[['test86b']]$result, pchisq(q=A$values,df=A$values,ncp=A$values,lower.tail=TRUE,log.p=TRUE), .001)
+omxCheckCloseEnough(model[['test87a']]$result, dchisq(x=D$values,df=1), .001)
+omxCheckCloseEnough(model[['test87b']]$result, pchisq(q=D$values,df=1,lower.tail=FALSE), .001)
+omxCheckCloseEnough(model[['test88a']]$result, dbinom(x=D$values,size=D$values,prob=A$values[1:10],log=FALSE), .001)
+omxCheckCloseEnough(model[['test88b']]$result, dbinom(x=D$values,size=D$values,prob=A$values[1:10],log=TRUE), .001)
+omxCheckCloseEnough(model[['test89a']]$result, pbinom(q=D$values,size=D$values,prob=A$values[1:10],lower.tail=FALSE,log.p=FALSE), .001)
+omxCheckCloseEnough(model[['test89b']]$result, pbinom(q=D$values,size=D$values,prob=A$values[1:10],lower.tail=TRUE,log.p=TRUE), .001)
+omxCheckCloseEnough(model[['test90a']]$result, dcauchy(x=H$values,location=A$values[1:16],scale=D$values[c(1:10,1:6)],log=FALSE), .001)
+omxCheckCloseEnough(model[['test90b']]$result, dcauchy(x=H$values,location=A$values[1:16],scale=D$values[c(1:10,1:6)],log=TRUE), .001)
+omxCheckCloseEnough(model[['test91a']]$result, pcauchy(q=H$values,location=A$values[1:16],scale=D$values[c(1:10,1:6)],lower.tail=FALSE,log=TRUE), 
+										.001)
+omxCheckCloseEnough(model[['test91b']]$result, pcauchy(q=H$values,location=A$values[1:16],scale=D$values[c(1:10,1:6)],lower.tail=TRUE,log=FALSE), 
+										.001)
 
 # Check internal function for definition variables
 m0 <- mxModel()
