@@ -1,5 +1,5 @@
  /*
- *  Copyright 2007-2016 The OpenMx Project
+ *  Copyright 2007-2017 The OpenMx Project
  *
  *  Licensed under the Apache License, Version 2.0 (the "License");
  *  you may not use this file except in compliance with the License.
@@ -22,6 +22,10 @@
 #include <Eigen/Cholesky>
 #include <Eigen/Dense>
  
+#ifdef SHADOW_DIAG
+#pragma GCC diagnostic warning "-Wshadow"
+#endif
+
 void omxInitGREMLExpectation(omxExpectation* ox){
   
   SEXP rObj = ox->rObj;
@@ -390,3 +394,44 @@ void dropCasesAndEigenize(omxMatrix* om, Eigen::MatrixXd &em, int num2drop, std:
   }
   if(OMX_DEBUG) { mxLog("Finished trimming out cases with missing data..."); }
 }
+
+
+
+void dropCasesFromAlgdV(omxMatrix* om, int num2drop, std::vector< int > todrop, int symmetric, int origDim){
+	
+	if(OMX_DEBUG) { mxLog("Trimming out cases with missing data..."); }
+	
+	if(num2drop < 1 || om->algebra == NULL){ return; }
+	
+	omxEnsureColumnMajor(om);
+	
+	if(origDim==0){Rf_error("Memory not allocated for algebra %s at downsize time",
+    om->name());}
+	if(om->rows != origDim || om->cols != origDim){
+		//Not sure if there are cases where this should be allowed
+		Rf_error("More than one attempt made to downsize algebra %s", om->name());
+		//return;
+	}
+	
+	int nextCol = 0;
+	int nextRow = 0;
+	
+	om->rows = origDim - num2drop;
+	om->cols = origDim - num2drop;
+	
+	for(int j = 0; j < origDim; j++){ //<--j indexes columns
+		if(todrop[j]) continue;
+		nextRow = (symmetric ? nextCol : 0);
+		for(int k = (symmetric ? j : 0); k < origDim; k++){ //<--k indexes rows
+			if(todrop[k]) continue;
+			omxSetMatrixElement(om, nextRow, nextCol, omxAliasedMatrixElement(om, k, j, origDim));
+			nextRow++;
+		}
+		nextCol++;
+	}
+	omxMarkDirty(om); //<--Need to mark it dirty so that it eventually gets recalculated back to original dimensions.
+	//^^^Algebras that do not depend upon free parameters, and upon which V does not depend, will not be
+	//recalculated back to full size until optimization is complete (the GREML fitfunction is smart about that).
+	if(OMX_DEBUG) { mxLog("Finished trimming out cases with missing data..."); }
+}
+
